@@ -125,16 +125,6 @@ def import_attrs(source, attrs, dest, prefix=''):
             setattr(dest, prefix + attr, attr_value)
 
 
-def serialise(element):
-    """Return `element` serialised, minus the tags for `element`."""
-    parts = []
-    parts.append(element.text or '')
-    for child in element:
-        parts.append(etree.tostring(
-            child, encoding='unicode', xml_declaration=False))
-    return ''.join(parts)
-
-
 def xpath(source, expression):
     return source.xpath(expression, namespaces=NS_MAP)
 
@@ -154,6 +144,9 @@ class Command(BaseCommand):
         xsd = self._get_xsd()
         parser = etree.XMLParser(ns_clean=True, remove_blank_text=True,
                                  schema=xsd)
+        xslt_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                                 'ead-to-html.xsl')
+        self._transform_to_html = etree.XSLT(etree.parse(xslt_path))
         for path in options['paths']:
             self._path = os.path.abspath(path)
             self._handle_path(self._path, parser)
@@ -186,26 +179,29 @@ class Command(BaseCommand):
         parent_obj.save()
 
     def _import_abstract(self, abstract, ead_obj):
-        abstract_obj = Abstract(did=ead_obj, abstract=serialise(abstract))
+        abstract_obj = Abstract(
+            did=ead_obj, abstract=self._serialise(abstract))
         import_attrs(abstract, ABSTRACT_ATTRS, abstract_obj)
         abstract_obj.full_clean()
         abstract_obj.save()
 
     def _import_accessrestrict(self, accessrestrict, ead_obj):
         accessrestrict_obj = AccessRestrict(
-            archdesc=ead_obj, accessrestrict=serialise(accessrestrict))
+            archdesc=ead_obj, accessrestrict=self._serialise(accessrestrict))
         import_attrs(accessrestrict, ACCESSRESTRICT_ATTRS, accessrestrict_obj)
         accessrestrict_obj.full_clean()
         accessrestrict_obj.save()
 
     def _import_accruals(self, accruals, ead_obj):
-        accruals_obj = Accruals(archdesc=ead_obj, accruals=serialise(accruals))
+        accruals_obj = Accruals(
+            archdesc=ead_obj, accruals=self._serialise(accruals))
         import_attrs(accruals, ACCRUALS_ATTRS, accruals_obj)
         accruals_obj.full_clean()
         accruals_obj.save()
 
     def _import_acqinfo(self, acqinfo, ead_obj):
-        acqinfo_obj = AcqInfo(archdesc=ead_obj, acqinfo=serialise(acqinfo))
+        acqinfo_obj = AcqInfo(
+            archdesc=ead_obj, acqinfo=self._serialise(acqinfo))
         import_attrs(acqinfo, ACQINFO_ATTRS, acqinfo_obj)
         acqinfo_obj.full_clean()
         acqinfo_obj.save()
@@ -219,7 +215,7 @@ class Command(BaseCommand):
 
     def _import_addressline(self, addressline, repository_obj):
         addressline_obj = RepositoryAddressLine(
-            address=repository_obj, addressline=serialise(addressline))
+            address=repository_obj, addressline=self._serialise(addressline))
         import_attrs(addressline, ADDRESSLINE_ATTRS, addressline_obj)
         addressline_obj.full_clean()
         addressline_obj.save()
@@ -248,14 +244,14 @@ class Command(BaseCommand):
 
     def _import_altformavail(self, altformavail, ead_obj):
         altformavail_obj = AltFormAvail(
-            archdesc=ead_obj, altformavail=serialise(altformavail))
+            archdesc=ead_obj, altformavail=self._serialise(altformavail))
         import_attrs(altformavail, ALTFORMAVAIL_ATTRS, altformavail_obj)
         altformavail_obj.full_clean()
         altformavail_obj.save()
 
     def _import_appraisal(self, appraisal, ead_obj):
         appraisal_obj = Appraisal(
-            archdesc=ead_obj, appraisal=serialise(appraisal))
+            archdesc=ead_obj, appraisal=self._serialise(appraisal))
         import_attrs(appraisal, APPRAISAL_ATTRS, appraisal_obj)
         appraisal_obj.full_clean()
         appraisal_obj.save()
@@ -315,36 +311,38 @@ class Command(BaseCommand):
 
     def _import_arrangement(self, arrangement, ead_obj):
         arrangement_obj = Arrangement(
-            archdesc=ead_obj, arrangement=serialise(arrangement))
+            archdesc=ead_obj, arrangement=self._serialise(arrangement))
         import_attrs(arrangement, ARRANGEMENT_ATTRS, arrangement_obj)
         arrangement_obj.full_clean()
         arrangement_obj.save()
 
     def _import_author(self, author, ead_obj):
-        author_obj = Author(titlestmt=ead_obj, author=serialise(author))
+        author_obj = Author(titlestmt=ead_obj, author=self._serialise(author))
         import_attrs(author, AUTHOR_ATTRS, author_obj)
         author_obj.full_clean()
         author_obj.save()
 
     def _import_bibliography(self, bibliography, ead_obj):
         bibliography_obj = Bibliography(
-            archdesc=ead_obj, bibliography=serialise(bibliography))
+            archdesc=ead_obj, bibliography=self._serialise(bibliography))
         import_attrs(bibliography, BIBLIOGRAPHY_ATTRS, bibliography_obj)
         bibliography_obj.full_clean()
         bibliography_obj.save()
 
     def _import_bioghist(self, bioghist, ead_obj):
-        bioghist_obj = BiogHist(archdesc=ead_obj, bioghist=serialise(bioghist))
+        bioghist_obj = BiogHist(
+            archdesc=ead_obj, bioghist=self._serialise(bioghist))
         import_attrs(bioghist, BIOGHIST_ATTRS, bioghist_obj)
         bioghist_obj.full_clean()
         bioghist_obj.save()
 
     def _import_citation(self, citation, parent_obj):
-        parent_obj.citation = serialise(citation)
+        parent_obj.citation = self._serialise(citation)
         import_attrs(citation, CITATION_ATTRS, parent_obj, 'citation_')
 
     def _import_container(self, container, ead_obj):
-        container_obj = Container(did=ead_obj, container=serialise(container))
+        container_obj = Container(
+            did=ead_obj, container=self._serialise(container))
         import_attrs(container, CONTAINER_ATTRS, container_obj)
         container_obj.full_clean()
         container_obj.save()
@@ -388,14 +386,14 @@ class Command(BaseCommand):
 
     def _import_controlaccess(self, controlaccess, ead_obj):
         controlaccess_obj = ControlAccess(
-            archdesc=ead_obj, controlaccess=serialise(controlaccess))
+            archdesc=ead_obj, controlaccess=self._serialise(controlaccess))
         import_attrs(controlaccess, CONTROLACCESS_ATTRS, controlaccess_obj)
         controlaccess_obj.full_clean()
         controlaccess_obj.save()
 
     def _import_controlnote(self, controlnote, ead_obj):
         controlnote_obj = ControlNote(
-            notestmt=ead_obj, controlnote=serialise(controlnote))
+            notestmt=ead_obj, controlnote=self._serialise(controlnote))
         import_attrs(controlnote, CONTROLNOTE_ATTRS, controlnote_obj)
         controlnote_obj.full_clean()
         controlnote_obj.save()
@@ -418,7 +416,7 @@ class Command(BaseCommand):
 
     def _import_custodhist(self, custodhist, ead_obj):
         custodhist_obj = CustodHist(
-            archdesc=ead_obj, custodhist=serialise(custodhist))
+            archdesc=ead_obj, custodhist=self._serialise(custodhist))
         import_attrs(custodhist, CUSTODHIST_ATTRS, custodhist_obj)
         custodhist_obj.full_clean()
         custodhist_obj.save()
@@ -459,12 +457,12 @@ class Command(BaseCommand):
         datesingle_obj = datesingle_model(parent=parent_obj)
         import_attrs(datesingle, DATESINGLE_ATTRS, datesingle_obj)
         if datesingle.text or len(datesingle):
-            datesingle_obj.datesingle = serialise(datesingle)
+            datesingle_obj.datesingle = self._serialise(datesingle)
         datesingle_obj.full_clean()
         datesingle_obj.save()
 
     def _import_descriptivenote(self, descriptivenote, parent_obj, prefix=''):
-        setattr(parent_obj, prefix + 'descriptivenote', serialise(
+        setattr(parent_obj, prefix + 'descriptivenote', self._serialise(
             descriptivenote))
         import_attrs(descriptivenote, DESCRIPTIVENOTE_ATTRS, parent_obj,
                      prefix + 'descriptivenote_')
@@ -521,14 +519,14 @@ class Command(BaseCommand):
             self._import_unittitle(unittitle, ead_obj)
 
     def _import_didnote(self, didnote, ead_obj):
-        didnote_obj = DIdNote(did=ead_obj, didnote=serialise(didnote))
+        didnote_obj = DIdNote(did=ead_obj, didnote=self._serialise(didnote))
         import_attrs(didnote, DIDNOTE_ATTRS, didnote_obj)
         didnote_obj.full_clean()
         didnote_obj.save()
 
     def _import_dimensions(self, dimensions, pds_obj, dimensions_model):
         dimensions_obj = dimensions_model(
-            physdescstructured=pds_obj, dimensions=serialise(dimensions))
+            physdescstructured=pds_obj, dimensions=self._serialise(dimensions))
         import_attrs(dimensions, DIMENSIONS_ATTRS, dimensions_obj)
         dimensions_obj.full_clean()
         dimensions_obj.save()
@@ -562,7 +560,7 @@ class Command(BaseCommand):
     def _import_editionstmt(self, editionstmt, ead_obj):
         import_attrs(editionstmt, EDITIONSTMT_ATTRS, ead_obj,
                      'editionstmt_')
-        ead_obj.editionstmt = serialise(editionstmt)
+        ead_obj.editionstmt = self._serialise(editionstmt)
 
     def _import_eventdatetime(self, eventdatetime, maintenanceevent_obj):
         import_attrs(eventdatetime, EVENTDATETIME_ATTRS, maintenanceevent_obj,
@@ -598,7 +596,8 @@ class Command(BaseCommand):
             self._import_notestmt(notestmt, ead_obj)
 
     def _import_fileplan(self, fileplan, ead_obj):
-        fileplan_obj = FilePlan(archdesc=ead_obj, fileplan=serialise(fileplan))
+        fileplan_obj = FilePlan(
+            archdesc=ead_obj, fileplan=self._serialise(fileplan))
         import_attrs(fileplan, FILEPLAN_ATTRS, fileplan_obj)
         fileplan_obj.full_clean()
         fileplan_obj.save()
@@ -608,20 +607,20 @@ class Command(BaseCommand):
         # for processing, store the attributes in model fields, along
         # with the serialised element.
         if fromtodate.text or len(fromtodate):
-            setattr(daterange_obj, field_name, serialise(fromtodate))
+            setattr(daterange_obj, field_name, self._serialise(fromtodate))
         import_attrs(fromtodate, FROMTODATE_ATTRS, daterange_obj,
                      field_name + '_')
 
     def _import_geogname(self, geogname, relation_obj):
-        relation_obj.geogname = serialise(geogname)
+        relation_obj.geogname = self._serialise(geogname)
         import_attrs(geogname, GEOGNAME_ATTRS, relation_obj, 'geogname_')
 
     def _import_head(self, head, parent_obj, prefix=''):
-        setattr(parent_obj, prefix + 'head', serialise(head))
+        setattr(parent_obj, prefix + 'head', self._serialise(head))
         import_attrs(head, HEAD_ATTRS, parent_obj, prefix + 'head_')
 
     def _import_index(self, index, ead_obj):
-        index_obj = Index(archdesc=ead_obj, index=serialise(index))
+        index_obj = Index(archdesc=ead_obj, index=self._serialise(index))
         import_attrs(index, INDEX_ATTRS, index_obj)
         index_obj.full_clean()
         index_obj.save()
@@ -683,7 +682,7 @@ class Command(BaseCommand):
 
     def _import_legalstatus(self, legalstatus, ead_obj):
         legalstatus_obj = LegalStatus(
-            archdesc=ead_obj, legalstatus=serialise(legalstatus))
+            archdesc=ead_obj, legalstatus=self._serialise(legalstatus))
         import_attrs(legalstatus, LEGALSTATUS_ATTRS, legalstatus_obj)
         legalstatus_obj.full_clean()
         legalstatus_obj.save()
@@ -704,7 +703,7 @@ class Command(BaseCommand):
             import_attrs(datesingle, DATESINGLE_ATTRS, localcontrol_obj,
                          'datesingle_')
             if datesingle.text or len(datesingle):
-                localcontrol_obj.datesingle = serialise(datesingle)
+                localcontrol_obj.datesingle = self._serialise(datesingle)
         localcontrol_obj.full_clean()
         localcontrol_obj.save()
 
@@ -764,7 +763,7 @@ class Command(BaseCommand):
 
     def _import_materialspec(self, materialspec, ead_obj):
         materialspec_obj = MaterialSpec(
-            did=ead_obj, materialspec=serialise(materialspec))
+            did=ead_obj, materialspec=self._serialise(materialspec))
         import_attrs(materialspec, MATERIALSPEC_ATTRS, materialspec_obj)
         materialspec_obj.full_clean()
         materialspec_obj.save()
@@ -794,17 +793,17 @@ class Command(BaseCommand):
     def _import_objectxmlwrap(self, objectxmlwrap, parent_obj):
         import_attrs(objectxmlwrap, OBJECTXMLWRAP_ATTRS, parent_obj,
                      'objectxmlwrap_')
-        parent_obj.objectxmlwrap = serialise(objectxmlwrap)
+        parent_obj.objectxmlwrap = self._serialise(objectxmlwrap, False)
 
     def _import_odd(self, odd, ead_obj):
-        odd_obj = ODD(archdesc=ead_obj, odd=serialise(odd))
+        odd_obj = ODD(archdesc=ead_obj, odd=self._serialise(odd))
         import_attrs(odd, ODD_ATTRS, odd_obj)
         odd_obj.full_clean()
         odd_obj.save()
 
     def _import_originalsloc(self, originalsloc, ead_obj):
         originalsloc_obj = OriginalsLoc(
-            archdesc=ead_obj, originalsloc=serialise(originalsloc))
+            archdesc=ead_obj, originalsloc=self._serialise(originalsloc))
         import_attrs(originalsloc, ORIGINALSLOC_ATTRS, originalsloc_obj)
         originalsloc_obj.full_clean()
         originalsloc_obj.save()
@@ -841,7 +840,7 @@ class Command(BaseCommand):
 
     def _import_otherfindaid(self, otherfindaid, ead_obj):
         otherfindaid_obj = OtherFindAid(
-            archdesc=ead_obj, otherfindaid=serialise(otherfindaid))
+            archdesc=ead_obj, otherfindaid=self._serialise(otherfindaid))
         import_attrs(otherfindaid, OTHERFINDAID_ATTRS, otherfindaid_obj)
         otherfindaid_obj.full_clean()
         otherfindaid_obj.save()
@@ -855,13 +854,14 @@ class Command(BaseCommand):
 
     def _import_part(self, part, name_obj, part_model, position):
         part_obj = part_model(
-            name=name_obj, part=serialise(part), order=position+1)
+            name=name_obj, part=self._serialise(part), order=position+1)
         import_attrs(part, PART_ATTRS, part_obj)
         part_obj.full_clean()
         part_obj.save()
 
     def _import_physdesc(self, physdesc, ead_obj):
-        physdesc_obj = PhysDesc(did=ead_obj, physdesc=serialise(physdesc))
+        physdesc_obj = PhysDesc(
+            did=ead_obj, physdesc=self._serialise(physdesc))
         import_attrs(physdesc, PHYSDESC_ATTRS, physdesc_obj)
         physdesc_obj.full_clean()
         physdesc_obj.save()
@@ -900,13 +900,13 @@ class Command(BaseCommand):
 
     def _import_physfacet(self, physfacet, pds_obj, physfacet_model):
         physfacet_obj = physfacet_model(
-            physdescstructured=pds_obj, physfacet=serialise(physfacet))
+            physdescstructured=pds_obj, physfacet=self._serialise(physfacet))
         import_attrs(physfacet, PHYSFACET_ATTRS, physfacet_obj)
         physfacet_obj.full_clean()
         physfacet_obj.save()
 
     def _import_physloc(self, physloc, ead_obj):
-        physloc_obj = PhysLoc(did=ead_obj, physloc=serialise(physloc))
+        physloc_obj = PhysLoc(did=ead_obj, physloc=self._serialise(physloc))
         import_attrs(physloc, PHYSLOC_ATTRS, physloc_obj)
         physloc_obj.full_clean()
         physloc_obj.save()
@@ -918,21 +918,22 @@ class Command(BaseCommand):
             physloc_obj.parents.add(parent_obj)
 
     def _import_phystech(self, phystech, ead_obj):
-        phystech_obj = PhysTech(archdesc=ead_obj, phystech=serialise(phystech))
+        phystech_obj = PhysTech(
+            archdesc=ead_obj, phystech=self._serialise(phystech))
         import_attrs(phystech, PHYSTECH_ATTRS, phystech_obj)
         phystech_obj.full_clean()
         phystech_obj.save()
 
     def _import_prefercite(self, prefercite, ead_obj):
         prefercite_obj = PreferCite(
-            archdesc=ead_obj, prefercite=serialise(prefercite))
+            archdesc=ead_obj, prefercite=self._serialise(prefercite))
         import_attrs(prefercite, PREFERCITE_ATTRS, prefercite_obj)
         prefercite_obj.full_clean()
         prefercite_obj.save()
 
     def _import_processinfo(self, processinfo, ead_obj):
         processinfo_obj = ProcessInfo(
-            archdesc=ead_obj, processinfo=serialise(processinfo))
+            archdesc=ead_obj, processinfo=self._serialise(processinfo))
         import_attrs(processinfo, PROCESSINFO_ATTRS, processinfo_obj)
         processinfo_obj.full_clean()
         processinfo_obj.save()
@@ -946,7 +947,7 @@ class Command(BaseCommand):
     def _import_publicationstmt(self, publicationstmt, ead_obj):
         import_attrs(publicationstmt, PUBLICATIONSTMT_ATTRS, ead_obj,
                      'publicationstmt_')
-        ead_obj.publicationstmt = serialise(publicationstmt)
+        ead_obj.publicationstmt = self._serialise(publicationstmt)
 
     def _import_quantity(self, quantity, physdescstructured_obj):
         physdescstructured_obj.quantity = quantity.text
@@ -959,7 +960,7 @@ class Command(BaseCommand):
 
     def _import_relatedmaterial(self, rm, ead_obj):
         rm_obj = RelatedMaterial(
-            archdesc=ead_obj, relatedmaterial=serialise(rm))
+            archdesc=ead_obj, relatedmaterial=self._serialise(rm))
         import_attrs(rm, RELATEDMATERIAL_ATTRS, rm_obj)
         rm_obj.full_clean()
         rm_obj.save()
@@ -1045,7 +1046,7 @@ class Command(BaseCommand):
 
     def _import_scopecontent(self, scopecontent, ead_obj):
         scopecontent_obj = ScopeContent(
-            archdesc=ead_obj, scopecontent=serialise(scopecontent))
+            archdesc=ead_obj, scopecontent=self._serialise(scopecontent))
         import_attrs(scopecontent, SCOPECONTENT_ATTRS, scopecontent_obj)
         scopecontent_obj.full_clean()
         scopecontent_obj.save()
@@ -1064,7 +1065,8 @@ class Command(BaseCommand):
 
     def _import_separatedmaterial(self, separatedmaterial, ead_obj):
         separatedmaterial_obj = SeparatedMaterial(
-            archdesc=ead_obj, separatedmaterial=serialise(separatedmaterial))
+            archdesc=ead_obj, separatedmaterial=self._serialise(
+                separatedmaterial))
         import_attrs(
             separatedmaterial, SEPARATEDMATERIAL_ATTRS, separatedmaterial_obj)
         separatedmaterial_obj.full_clean()
@@ -1072,7 +1074,7 @@ class Command(BaseCommand):
 
     def _import_seriesstmt(self, seriesstmt, ead_obj):
         import_attrs(seriesstmt, SERIESSTMT_ATTRS, ead_obj, 'seriesstmt_')
-        ead_obj.seriesstmt = serialise(seriesstmt)
+        ead_obj.seriesstmt = self._serialise(seriesstmt)
 
     def _import_source(self, source, ead_obj):
         source_obj = Source(sources=ead_obj)
@@ -1100,14 +1102,15 @@ class Command(BaseCommand):
             self._import_source(source, ead_obj)
 
     def _import_sponsor(self, sponsor, ead_obj):
-        sponsor_obj = Sponsor(titlestmt=ead_obj, sponsor=serialise(sponsor))
+        sponsor_obj = Sponsor(
+            titlestmt=ead_obj, sponsor=self._serialise(sponsor))
         import_attrs(sponsor, SPONSOR_ATTRS, sponsor_obj)
         sponsor_obj.full_clean()
         sponsor_obj.save()
 
     def _import_subtitle(self, subtitle, ead_obj):
         subtitle_obj = Subtitle(
-            titlestmt=ead_obj, subtitle=serialise(subtitle))
+            titlestmt=ead_obj, subtitle=self._serialise(subtitle))
         import_attrs(subtitle, SUBTITLE_ATTRS, subtitle_obj)
         subtitle_obj.full_clean()
         subtitle_obj.save()
@@ -1120,7 +1123,7 @@ class Command(BaseCommand):
         # Use only for titleproper within titlestmt; within
         # seriesstmt, no new model object is created.
         titleproper_obj = TitleProper(
-            titlestmt=ead_obj, titleproper=serialise(titleproper))
+            titlestmt=ead_obj, titleproper=self._serialise(titleproper))
         import_attrs(titleproper, TITLEPROPER_ATTRS, titleproper_obj)
         titleproper_obj.full_clean()
         titleproper_obj.save()
@@ -1137,7 +1140,8 @@ class Command(BaseCommand):
             self._import_sponsor(sponsor, ead_obj)
 
     def _import_unitdate(self, unitdate, ead_obj):
-        unitdate_obj = UnitDate(did=ead_obj, unitdate=serialise(unitdate))
+        unitdate_obj = UnitDate(
+            did=ead_obj, unitdate=self._serialise(unitdate))
         import_attrs(unitdate, UNITDATE_ATTRS, unitdate_obj)
         unitdate_obj.full_clean()
         unitdate_obj.save()
@@ -1157,13 +1161,14 @@ class Command(BaseCommand):
                 datesingle, uds_obj, UnitDateStructuredDateSingle)
 
     def _import_unitid(self, unitid, ead_obj):
-        unitid_obj = UnitId(did=ead_obj, unitid=serialise(unitid))
+        unitid_obj = UnitId(did=ead_obj, unitid=self._serialise(unitid))
         import_attrs(unitid, UNITID_ATTRS, unitid_obj)
         unitid_obj.full_clean()
         unitid_obj.save()
 
     def _import_unittitle(self, unittitle, ead_obj):
-        unittitle_obj = UnitTitle(did=ead_obj, unittitle=serialise(unittitle))
+        unittitle_obj = UnitTitle(
+            did=ead_obj, unittitle=self._serialise(unittitle))
         import_attrs(unittitle, UNITTITLE_ATTRS, unittitle_obj)
         unittitle_obj.full_clean()
         unittitle_obj.save()
@@ -1175,7 +1180,25 @@ class Command(BaseCommand):
 
     def _import_userestrict(self, userestrict, ead_obj):
         userestrict_obj = UseRestrict(
-            archdesc=ead_obj, userestrict=serialise(userestrict))
+            archdesc=ead_obj, userestrict=self._serialise(userestrict))
         import_attrs(userestrict, USERESTRICT_ATTRS, userestrict_obj)
         userestrict_obj.full_clean()
         userestrict_obj.save()
+
+    def _serialise(self, element, to_html=True):
+        """Return `element` serialised, minus the tags for `element`."""
+        if to_html:
+            element = self._transform(element)
+        parts = []
+        parts.append(element.text or '')
+        for child in element:
+            parts.append(etree.tostring(
+                child, encoding='unicode', xml_declaration=False))
+        return ''.join(parts)
+
+    def _transform(self, element):
+        """Return `element` transformed from EAD3 into HTML with data-*
+        attributes to allow exact transformation back into EAD3."""
+        root = etree.fromstring(etree.tostring(element))
+        result = self._transform_to_html(root)
+        return result.getroot()
