@@ -483,7 +483,10 @@ class DimensionsMixin(CommonMixin, EncodingAnalogMixin, LangScriptMixin,
 
 class LanguageMixin(models.Model):
     """Corresponds to the text content of EAD3 language."""
-    language = models.CharField(max_length=32)
+    language = models.CharField(blank=True, max_length=32)
+
+    class Meta:
+        abstract = True
 
 
 class NameMixin(AccessMixin, CommonMixin, EncodingAnalogMixin,
