@@ -1,3 +1,5 @@
+import os.path
+
 from lxml import etree
 
 from .constants import (
@@ -36,14 +38,6 @@ from .constants import (
 NS_MAP = {None: EAD_NAMESPACE}
 
 
-def append_xml(parent, xml):
-    if xml:
-        root = etree.fromstring('<wrapper>{}</wrapper>'.format(xml))
-        parent.text = root.text
-        for child in root:
-            parent.append(child)
-
-
 def serialise_attrs(obj, attrs, element, prefix=''):
     for attr in attrs:
         if attr == 'id':
@@ -64,6 +58,20 @@ def serialise_attrs(obj, attrs, element, prefix=''):
 
 class EADSerialiser:
 
+    def __init__(self):
+        xslt_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                                 'management', 'commands', 'html-to-ead.xsl')
+        self._transform_to_ead = etree.XSLT(etree.parse(xslt_path))
+
+    def _append_xml(self, parent, xml, from_html=True):
+        if xml:
+            root = etree.fromstring('<wrapper>{}</wrapper>'.format(xml))
+            if from_html:
+                root = self._transform_to_ead(root).getroot()
+            parent.text = root.text
+            for child in root:
+                parent.append(child)
+
     def serialise(self, ead_obj):
         ead = self.to_xml(ead_obj)
         return etree.tostring(ead, encoding='unicode', pretty_print=True)
@@ -85,28 +93,28 @@ class EADSerialiser:
     def _serialise_abstract(self, abstract_obj, did):
         abstract = etree.SubElement(did, EAD + 'abstract')
         serialise_attrs(abstract_obj, ABSTRACT_ATTRS, abstract)
-        append_xml(abstract, abstract_obj.abstract)
+        self._append_xml(abstract, abstract_obj.abstract)
 
     def _serialise_accessrestrict(self, accessrestrict_obj, archdesc):
         accessrestrict = etree.SubElement(archdesc, EAD + 'accessrestrict')
         serialise_attrs(
             accessrestrict_obj, ACCESSRESTRICT_ATTRS, accessrestrict)
-        append_xml(accessrestrict, accessrestrict_obj.accessrestrict)
+        self._append_xml(accessrestrict, accessrestrict_obj.accessrestrict)
 
     def _serialise_accruals(self, accruals_obj, archdesc):
         accruals = etree.SubElement(archdesc, EAD + 'accruals')
         serialise_attrs(accruals_obj, ACCRUALS_ATTRS, accruals)
-        append_xml(accruals, accruals_obj.accruals)
+        self._append_xml(accruals, accruals_obj.accruals)
 
     def _serialise_acqinfo(self, acqinfo_obj, archdesc):
         acqinfo = etree.SubElement(archdesc, EAD + 'acqinfo')
         serialise_attrs(acqinfo_obj, ACQINFO_ATTRS, acqinfo)
-        append_xml(acqinfo, acqinfo_obj.acqinfo)
+        self._append_xml(acqinfo, acqinfo_obj.acqinfo)
 
     def _serialise_addressline(self, addressline_obj, address):
         addressline = etree.SubElement(address, EAD + 'addressline')
         serialise_attrs(addressline_obj, ADDRESSLINE_ATTRS, addressline)
-        append_xml(addressline, addressline_obj.addressline)
+        self._append_xml(addressline, addressline_obj.addressline)
 
     def _serialise_agencycode(self, ead_obj, maintenanceagency):
         if ead_obj.agencycode:
@@ -135,12 +143,12 @@ class EADSerialiser:
     def _serialise_altformavail(self, altformavail_obj, archdesc):
         altformavail = etree.SubElement(archdesc, EAD + 'altformavail')
         serialise_attrs(altformavail_obj, ALTFORMAVAIL_ATTRS, altformavail)
-        append_xml(altformavail, altformavail_obj.altformavail)
+        self._append_xml(altformavail, altformavail_obj.altformavail)
 
     def _serialise_appraisal(self, appraisal_obj, archdesc):
         appraisal = etree.SubElement(archdesc, EAD + 'appraisal')
         serialise_attrs(appraisal_obj, APPRAISAL_ATTRS, appraisal)
-        append_xml(appraisal, appraisal_obj.appraisal)
+        self._append_xml(appraisal, appraisal_obj.appraisal)
 
     def _serialise_archdesc(self, ead_obj, ead):
         archdesc = etree.SubElement(ead, EAD + 'archdesc')
@@ -197,27 +205,27 @@ class EADSerialiser:
     def _serialise_arrangement(self, arrangement_obj, archdesc):
         arrangement = etree.SubElement(archdesc, EAD + 'arrangement')
         serialise_attrs(arrangement_obj, ARRANGEMENT_ATTRS, arrangement)
-        append_xml(arrangement, arrangement_obj.arrangement)
+        self._append_xml(arrangement, arrangement_obj.arrangement)
 
     def _serialise_author(self, author_obj, titlestmt):
         author = etree.SubElement(titlestmt, EAD + 'author')
         serialise_attrs(author_obj, AUTHOR_ATTRS, author)
-        append_xml(author, author_obj.author)
+        self._append_xml(author, author_obj.author)
 
     def _serialise_bibliography(self, bibliography_obj, archdesc):
         bibliography = etree.SubElement(archdesc, EAD + 'bibliography')
         serialise_attrs(bibliography_obj, BIBLIOGRAPHY_ATTRS, bibliography)
-        append_xml(bibliography, bibliography_obj.bibliography)
+        self._append_xml(bibliography, bibliography_obj.bibliography)
 
     def _serialise_bioghist(self, bioghist_obj, archdesc):
         bioghist = etree.SubElement(archdesc, EAD + 'bioghist')
         serialise_attrs(bioghist_obj, BIOGHIST_ATTRS, bioghist)
-        append_xml(bioghist, bioghist_obj.bioghist)
+        self._append_xml(bioghist, bioghist_obj.bioghist)
 
     def _serialise_citation(self, parent_obj, parent):
         citation = etree.SubElement(parent, EAD + 'citation')
         serialise_attrs(parent_obj, CITATION_ATTRS, citation, 'citation_')
-        append_xml(citation, parent_obj.citation)
+        self._append_xml(citation, parent_obj.citation)
 
     def _serialise_container(self, container_obj, did):
         container = etree.SubElement(did, EAD + 'container')
@@ -225,7 +233,7 @@ class EADSerialiser:
         parents = [parent.ead_id for parent in container_obj.parents.all()]
         if parents:
             container.set('parent', ' '.join(parents))
-        append_xml(container, container_obj.container)
+        self._append_xml(container, container_obj.container)
 
     def _serialise_control(self, ead_obj, ead):
         control = etree.SubElement(ead, EAD + 'control')
@@ -255,12 +263,12 @@ class EADSerialiser:
     def _serialise_controlaccess(self, controlaccess_obj, archdesc):
         controlaccess = etree.SubElement(archdesc, EAD + 'controlaccess')
         serialise_attrs(controlaccess_obj, CONTROLACCESS_ATTRS, controlaccess)
-        append_xml(controlaccess, controlaccess_obj.controlaccess)
+        self._append_xml(controlaccess, controlaccess_obj.controlaccess)
 
     def _serialise_controlnote(self, controlnote_obj, notestmt):
         controlnote = etree.SubElement(notestmt, EAD + 'controlnote')
         serialise_attrs(controlnote_obj, CONTROLNOTE_ATTRS, controlnote)
-        append_xml(controlnote, controlnote_obj.controlnote)
+        self._append_xml(controlnote, controlnote_obj.controlnote)
 
     def _serialise_conventiondeclaration(self, conventiondeclaration_obj,
                                          control):
@@ -277,7 +285,7 @@ class EADSerialiser:
     def _serialise_custodhist(self, custodhist_obj, archdesc):
         custodhist = etree.SubElement(archdesc, EAD + 'custodhist')
         serialise_attrs(custodhist_obj, CUSTODHIST_ATTRS, custodhist)
-        append_xml(custodhist, custodhist_obj.custodhist)
+        self._append_xml(custodhist, custodhist_obj.custodhist)
 
     def _serialise_dao(self, dao_obj, parent):
         dao = etree.SubElement(parent, EAD + 'dao')
@@ -310,7 +318,7 @@ class EADSerialiser:
     def _serialise_datesingle(self, datesingle_obj, parent, prefix=''):
         datesingle = etree.SubElement(parent, EAD + 'datesingle')
         serialise_attrs(datesingle_obj, DATESINGLE_ATTRS, datesingle, prefix)
-        append_xml(datesingle, datesingle_obj.datesingle)
+        self._append_xml(datesingle, datesingle_obj.datesingle)
 
     def _serialise_descriptivenote(self, obj, parent, prefix=''):
         content = getattr(obj, prefix + 'descriptivenote')
@@ -319,7 +327,7 @@ class EADSerialiser:
         descriptivenote = etree.SubElement(parent, EAD + 'descriptivenote')
         serialise_attrs(obj, DESCRIPTIVENOTE_ATTRS, descriptivenote,
                         prefix + 'descriptivenote_')
-        append_xml(descriptivenote, content)
+        self._append_xml(descriptivenote, content)
 
     def _serialise_did(self, ead_obj, archdesc):
         did = etree.SubElement(archdesc, EAD + 'did')
@@ -363,12 +371,12 @@ class EADSerialiser:
     def _serialise_didnote(self, didnote_obj, did):
         didnote = etree.SubElement(did, EAD + 'didnote')
         serialise_attrs(didnote_obj, DIDNOTE_ATTRS, didnote)
-        append_xml(didnote, didnote_obj.didnote)
+        self._append_xml(didnote, didnote_obj.didnote)
 
     def _serialise_dimensions(self, dimensions_obj, pds):
         dimensions = etree.SubElement(pds, EAD + 'dimensions')
         serialise_attrs(dimensions_obj, DIMENSIONS_ATTRS, dimensions)
-        append_xml(dimensions, dimensions_obj.dimensions)
+        self._append_xml(dimensions, dimensions_obj.dimensions)
 
     def _serialise_editionstmt(self, ead_obj, filedesc):
         if not ead_obj.editionstmt:
@@ -376,7 +384,7 @@ class EADSerialiser:
         editionstmt = etree.SubElement(filedesc, EAD + 'editionstmt')
         serialise_attrs(ead_obj, EDITIONSTMT_ATTRS, editionstmt,
                         'editionstmt_')
-        append_xml(editionstmt, ead_obj.editionstmt)
+        self._append_xml(editionstmt, ead_obj.editionstmt)
 
     def _serialise_eventdatetime(self, maintenanceevent_obj, maintenanceevent):
         eventdatetime = etree.SubElement(
@@ -409,7 +417,7 @@ class EADSerialiser:
     def _serialise_fileplan(self, fileplan_obj, archdesc):
         fileplan = etree.SubElement(archdesc, EAD + 'fileplan')
         serialise_attrs(fileplan_obj, FILEPLAN_ATTRS, fileplan)
-        append_xml(fileplan, fileplan_obj.fileplan)
+        self._append_xml(fileplan, fileplan_obj.fileplan)
 
     def _serialise_fromtodate(self, daterange_obj, daterange, element_name):
         date = etree.SubElement(daterange, EAD + element_name)
@@ -418,14 +426,14 @@ class EADSerialiser:
         # serialised string.
         serialise_attrs(
             daterange_obj, FROMTODATE_ATTRS, date, element_name + '_')
-        append_xml(date, getattr(daterange_obj, element_name))
+        self._append_xml(date, getattr(daterange_obj, element_name))
 
     def _serialise_geogname(self, relation_obj, relation):
         if not relation_obj.geogname:
             return
         geogname = etree.SubElement(relation, EAD + 'geogname')
         serialise_attrs(relation_obj, GEOGNAME_ATTRS, geogname, 'geogname_')
-        append_xml(geogname, relation_obj.geogname)
+        self._append_xml(geogname, relation_obj.geogname)
 
     def _serialise_head(self, obj, parent, prefix=''):
         content = getattr(obj, prefix + 'head')
@@ -433,12 +441,12 @@ class EADSerialiser:
             return
         head = etree.SubElement(parent, EAD + 'head')
         serialise_attrs(obj, HEAD_ATTRS, head, prefix + 'head_')
-        append_xml(head, content)
+        self._append_xml(head, content)
 
     def _serialise_index(self, index_obj, archdesc):
         index = etree.SubElement(archdesc, EAD + 'index')
         serialise_attrs(index_obj, INDEX_ATTRS, index)
-        append_xml(index, index_obj.index)
+        self._append_xml(index, index_obj.index)
 
     def _serialise_langmaterial(self, langmaterial_obj, did):
         langmaterial = etree.SubElement(did, EAD + 'langmaterial')
@@ -478,7 +486,7 @@ class EADSerialiser:
     def _serialise_legalstatus(self, legalstatus_obj, archdesc):
         legalstatus = etree.SubElement(archdesc, EAD + 'legalstatus')
         serialise_attrs(legalstatus_obj, LEGALSTATUS_ATTRS, legalstatus)
-        append_xml(legalstatus, legalstatus_obj.legalstatus)
+        self._append_xml(legalstatus, legalstatus_obj.legalstatus)
 
     def _serialise_localcontrol(self, localcontrol_obj, control):
         localcontrol = etree.SubElement(control, EAD + 'localcontrol')
@@ -548,7 +556,7 @@ class EADSerialiser:
     def _serialise_materialspec(self, materialspec_obj, did):
         materialspec = etree.SubElement(did, EAD + 'materialspec')
         serialise_attrs(materialspec_obj, MATERIALSPEC_ATTRS, materialspec)
-        append_xml(materialspec, materialspec_obj.materialspec)
+        self._append_xml(materialspec, materialspec_obj.materialspec)
 
     def _serialise_name(self, name_obj, parent, element_name):
         name = etree.SubElement(parent, EAD + element_name)
@@ -570,17 +578,17 @@ class EADSerialiser:
         objectxmlwrap = etree.SubElement(parent, EAD + 'objectxmlwrap')
         serialise_attrs(obj, OBJECTXMLWRAP_ATTRS, objectxmlwrap,
                         'objectxmlwrap_')
-        append_xml(objectxmlwrap, obj.objectxmlwrap)
+        self._append_xml(objectxmlwrap, obj.objectxmlwrap, False)
 
     def _serialise_odd(self, odd_obj, archdesc):
         odd = etree.SubElement(archdesc, EAD + 'odd')
         serialise_attrs(odd_obj, ODD_ATTRS, odd)
-        append_xml(odd, odd_obj.odd)
+        self._append_xml(odd, odd_obj.odd)
 
     def _serialise_originalsloc(self, originalsloc_obj, archdesc):
         originalsloc = etree.SubElement(archdesc, EAD + 'originalsloc')
         serialise_attrs(originalsloc_obj, ORIGINALSLOC_ATTRS, originalsloc)
-        append_xml(originalsloc, originalsloc_obj.originalsloc)
+        self._append_xml(originalsloc, originalsloc_obj.originalsloc)
 
     def _serialise_origination(self, origination_obj, did):
         origination = etree.SubElement(did, EAD + 'origination')
@@ -605,7 +613,7 @@ class EADSerialiser:
     def _serialise_otherfindaid(self, otherfindaid_obj, archdesc):
         otherfindaid = etree.SubElement(archdesc, EAD + 'otherfindaid')
         serialise_attrs(otherfindaid_obj, OTHERFINDAID_ATTRS, otherfindaid)
-        append_xml(otherfindaid, otherfindaid_obj.otherfindaid)
+        self._append_xml(otherfindaid, otherfindaid_obj.otherfindaid)
 
     def _serialise_otherrecordid(self, otherrecordid_obj, control):
         otherrecordid = etree.SubElement(control, EAD + 'otherrecordid')
@@ -615,12 +623,12 @@ class EADSerialiser:
     def _serialise_part(self, part_obj, name):
         part = etree.SubElement(name, EAD + 'part')
         serialise_attrs(part_obj, PART_ATTRS, part)
-        append_xml(part, part_obj.part)
+        self._append_xml(part, part_obj.part)
 
     def _serialise_physdesc(self, physdesc_obj, did):
         physdesc = etree.SubElement(did, EAD + 'physdesc')
         serialise_attrs(physdesc_obj, PHYSDESC_ATTRS, physdesc)
-        append_xml(physdesc, physdesc_obj.physdesc)
+        self._append_xml(physdesc, physdesc_obj.physdesc)
 
     def _serialise_physdescset(self, physdescset_obj, did):
         physdescset = etree.SubElement(did, EAD + 'physdescset')
@@ -642,7 +650,7 @@ class EADSerialiser:
     def _serialise_physfacet(self, physfacet_obj, physdescstructured):
         physfacet = etree.SubElement(physdescstructured, EAD + 'physfacet')
         serialise_attrs(physfacet_obj, PHYSFACET_ATTRS, physfacet)
-        append_xml(physfacet, physfacet_obj.physfacet)
+        self._append_xml(physfacet, physfacet_obj.physfacet)
 
     def _serialise_physloc(self, physloc_obj, did):
         physloc = etree.SubElement(did, EAD + 'physloc')
@@ -650,22 +658,22 @@ class EADSerialiser:
         parents = [parent.ead_id for parent in physloc_obj.parents.all()]
         if parents:
             physloc.set('parent', ' '.join(parents))
-        append_xml(physloc, physloc_obj.physloc)
+        self._append_xml(physloc, physloc_obj.physloc)
 
     def _serialise_phystech(self, phystech_obj, archdesc):
         phystech = etree.SubElement(archdesc, EAD + 'phystech')
         serialise_attrs(phystech_obj, PHYSTECH_ATTRS, phystech)
-        append_xml(phystech, phystech_obj.phystech)
+        self._append_xml(phystech, phystech_obj.phystech)
 
     def _serialise_prefercite(self, prefercite_obj, archdesc):
         prefercite = etree.SubElement(archdesc, EAD + 'prefercite')
         serialise_attrs(prefercite_obj, PREFERCITE_ATTRS, prefercite)
-        append_xml(prefercite, prefercite_obj.prefercite)
+        self._append_xml(prefercite, prefercite_obj.prefercite)
 
     def _serialise_processinfo(self, processinfo_obj, archdesc):
         processinfo = etree.SubElement(archdesc, EAD + 'processinfo')
         serialise_attrs(processinfo_obj, PROCESSINFO_ATTRS, processinfo)
-        append_xml(processinfo, processinfo_obj.processinfo)
+        self._append_xml(processinfo, processinfo_obj.processinfo)
 
     def _serialise_publicationstatus(self, ead_obj, control):
         status = ead_obj.publicationstatus_value
@@ -681,7 +689,7 @@ class EADSerialiser:
         publicationstmt = etree.SubElement(filedesc, EAD + 'publicationstmt')
         serialise_attrs(ead_obj, PUBLICATIONSTMT_ATTRS, publicationstmt,
                         'publicationstmt_')
-        append_xml(publicationstmt, ead_obj.publicationstmt)
+        self._append_xml(publicationstmt, ead_obj.publicationstmt)
 
     def _serialise_quantity(self, physdescstructured_obj, pds):
         if not physdescstructured_obj.quantity:
@@ -701,7 +709,7 @@ class EADSerialiser:
         relatedmaterial = etree.SubElement(archdesc, EAD + 'relatedmaterial')
         serialise_attrs(
             relatedmaterial_obj, RELATEDMATERIAL_ATTRS, relatedmaterial)
-        append_xml(relatedmaterial, relatedmaterial_obj.relatedmaterial)
+        self._append_xml(relatedmaterial, relatedmaterial_obj.relatedmaterial)
 
     def _serialise_relation(self, relation_obj, relations):
         relation = etree.SubElement(relations, EAD + 'relation')
@@ -773,7 +781,7 @@ class EADSerialiser:
     def _serialise_scopecontent(self, scopecontent_obj, parent):
         scopecontent = etree.SubElement(parent, EAD + 'scopecontent')
         serialise_attrs(scopecontent_obj, SCOPECONTENT_ATTRS, scopecontent)
-        append_xml(scopecontent, scopecontent_obj.scopecontent)
+        self._append_xml(scopecontent, scopecontent_obj.scopecontent)
 
     def _serialise_script(self, obj, parent, prefix=''):
         scriptcode = getattr(obj, prefix + 'scriptcode')
@@ -788,14 +796,14 @@ class EADSerialiser:
     def _serialise_separatedmaterial(self, sm_obj, archdesc):
         sm = etree.SubElement(archdesc, EAD + 'separatedmaterial')
         serialise_attrs(sm_obj, SEPARATEDMATERIAL_ATTRS, sm)
-        append_xml(sm, sm_obj.separatedmaterial)
+        self._append_xml(sm, sm_obj.separatedmaterial)
 
     def _serialise_seriesstmt(self, ead_obj, filedesc):
         if not ead_obj.seriesstmt:
             return
         seriesstmt = etree.SubElement(filedesc, EAD + 'seriesstmt')
         serialise_attrs(ead_obj, SERIESSTMT_ATTRS, seriesstmt, 'seriesstmt_')
-        append_xml(seriesstmt, ead_obj.seriesstmt)
+        self._append_xml(seriesstmt, ead_obj.seriesstmt)
 
     def _serialise_source(self, source_obj, sources):
         source = etree.SubElement(sources, EAD + 'source')
@@ -822,12 +830,12 @@ class EADSerialiser:
     def _serialise_sponsor(self, sponsor_obj, titlestmt):
         sponsor = etree.SubElement(titlestmt, EAD + 'sponsor')
         serialise_attrs(sponsor_obj, SPONSOR_ATTRS, sponsor)
-        append_xml(sponsor, sponsor_obj.sponsor)
+        self._append_xml(sponsor, sponsor_obj.sponsor)
 
     def _serialise_subtitle(self, subtitle_obj, titlestmt):
         subtitle = etree.SubElement(titlestmt, EAD + 'subtitle')
         serialise_attrs(subtitle_obj, SUBTITLE_ATTRS, subtitle)
-        append_xml(subtitle, subtitle_obj.subtitle)
+        self._append_xml(subtitle, subtitle_obj.subtitle)
 
     def _serialise_term(self, localcontrol_obj, localcontrol):
         if localcontrol_obj.term:
@@ -838,7 +846,7 @@ class EADSerialiser:
     def _serialise_titleproper(self, titleproper_obj, parent):
         titleproper = etree.SubElement(parent, EAD + 'titleproper')
         serialise_attrs(titleproper_obj, TITLEPROPER_ATTRS, titleproper)
-        append_xml(titleproper, titleproper_obj.titleproper)
+        self._append_xml(titleproper, titleproper_obj.titleproper)
 
     def _serialise_titlestmt(self, ead_obj, filedesc):
         titlestmt = etree.SubElement(filedesc, EAD + 'titlestmt')
@@ -855,7 +863,7 @@ class EADSerialiser:
     def _serialise_unitdate(self, unitdate_obj, did):
         unitdate = etree.SubElement(did, EAD + 'unitdate')
         serialise_attrs(unitdate_obj, UNITDATE_ATTRS, unitdate)
-        append_xml(unitdate, unitdate_obj.unitdate)
+        self._append_xml(unitdate, unitdate_obj.unitdate)
 
     def _serialise_unitdatestructured(self, unitdatestructured_obj, did):
         unitdatestructured = etree.SubElement(did, EAD + 'unitdatestructured')
@@ -876,12 +884,12 @@ class EADSerialiser:
     def _serialise_unitid(self, unitid_obj, did):
         unitid = etree.SubElement(did, EAD + 'unitid')
         serialise_attrs(unitid_obj, UNITID_ATTRS, unitid)
-        append_xml(unitid, unitid_obj.unitid)
+        self._append_xml(unitid, unitid_obj.unitid)
 
     def _serialise_unittitle(self, unittitle_obj, did):
         unittitle = etree.SubElement(did, EAD + 'unittitle')
         serialise_attrs(unittitle_obj, UNITTITLE_ATTRS, unittitle)
-        append_xml(unittitle, unittitle_obj.unittitle)
+        self._append_xml(unittitle, unittitle_obj.unittitle)
 
     def _serialise_unittype(self, physdescstructured_obj, pds):
         if not physdescstructured_obj.unittype:
@@ -894,4 +902,4 @@ class EADSerialiser:
     def _serialise_userestrict(self, userestrict_obj, archdesc):
         userestrict = etree.SubElement(archdesc, EAD + 'userestrict')
         serialise_attrs(userestrict_obj, USERESTRICT_ATTRS, userestrict)
-        append_xml(userestrict, userestrict_obj.userestrict)
+        self._append_xml(userestrict, userestrict_obj.userestrict)
